#!/bin/bash

cd `dirname $0`
MYDIR=`pwd`
MYSCRIPT=`basename $0`
cd ../..
S2P_PATH_BASE=`pwd`
cd $MYDIR

COMPILE_PATH=/usr/src
S2P_HQ_SYSNAME=`$S2P_PATH_BASE/core/libexec/config_view.py system simple | grep ^hq.sysname | cut -d= -f2`

COMPILE_KERNEL=$1

if [ -z "$COMPILE_KERNEL" ]; then
	export COMPILE_KERNEL=`uname -r`
fi

export DIALOG_TITLE=":: $S2P_HQ_SYSNAME third party components ::"

apt-get install -y -qq --allow-unauthenticated linux-headers-$COMPILE_KERNEL &> /dev/null
cp -Rfp $S2P_PATH_BASE/core/3rdparty/*.tar.gz $COMPILE_PATH &> /dev/null

display_error()
{
	dialog --title "Error!!" --backtitle "$DIALOG_TITLE" --exit-label "Exit" --msgbox "\n$1" 12 70
	exit -1
}

display_info_extracting()
{
	dialog --title "Extracting $1" --backtitle "$DIALOG_TITLE" --infobox "\n$1: EXTRACTING\n\nThis operation may require several minutes." 12 70
}

display_info_configuring()
{
	dialog --title "Configuring $1" --backtitle "$DIALOG_TITLE" --infobox "\n$1: CONFIGURING\n\nThis operation may require several minutes." 12 70
}

display_info_compiling()
{
	dialog --title "Compiling $1" --backtitle "$DIALOG_TITLE" --infobox "\n$1: COMPILING\n\nThis operation may require several minutes." 12 70
}

display_info_installing()
{
	dialog --title "Installing $1"  --backtitle "$DIALOG_TITLE" --infobox "\n$1: INSTALLING\n\nThis operation may require several minutes." 12 70
}

install_zaptel()
{
	cd $COMPILE_PATH

	export KVERS=$COMPILE_KERNEL

	display_info_extracting "[$1] Zaptel"
	(tar -xzf zaptel.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf zaptel.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH
	
	display_info_configuring "[$1] Zaptel"
	(./configure --prefix=/usr &> /dev/null) || return 0
	
	display_info_compiling "[$1] Zaptel"
	(make &> /dev/null) || return 0
	(make utils &> /dev/null) || return 0

	display_info_installing "[$1] Zaptel"
	(make install &> /dev/null) || return 0

	return 1
}

install_libzap()
{
	cd $COMPILE_PATH

	export KVERS=$COMPILE_KERNEL

	display_info_extracting "[$1] Libzap"
	(tar -xzf zapata.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf zapata.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_compiling "[$1] Libzap"
	(make &> /dev/null) || return 0

	display_info_installing "[$1] Libzap"
	(make install &> /dev/null) || return 0

	return 1
}

install_libpri()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] Libpri"
	(tar -xzf libpri.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf libpri.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_compiling "[$1] Libpri"
	(make &> /dev/null) || return 0

	display_info_installing "[$1] Libpri"
	(make install &> /dev/null) || return 0

	return 1
}

install_wanpipe()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] Sangoma wanpipe"
	(tar -xzf wanpipe.tar.gz &> /dev/null) || return 0
	
	SUBPATH=`tar -tzf wanpipe.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_installing "[$1] Sangoma wanpipe"
	(bash ./Setup install --with-linux=/lib/modules/$COMPILE_KERNEL/build --no-zaptel-compile --silent --protocols=TDM &> /dev/null) || return 0

	return 1
}

install_divas()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] Divas"
	(tar -xzf divas4linux.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf divas4linux.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	KDIR=/lib/modules/$COMPILE_KERNEL/build

	display_info_compiling "[$1] Divas"
	(echo "y" | make KDIR=$KDIR &> /dev/null) || return 0

	display_info_installing "[$1] Divas"
	(echo "y" | make install KDIR=$KDIR &> /dev/null) || return 0

	return 1
}

install_osip()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] oSIP"
	(tar -xzf libosip2.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf libosip2.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_configuring "[$1] oSIP"
	(./configure --prefix=/usr &> /dev/null) || return 0
	
	display_info_compiling "[$1] oSIP"
	(make &> /dev/null) || return 0

	display_info_installing "[$1] oSIP"
	(make install &> /dev/null) || return 0

	return 1
}

install_exosip()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] eXosip"
	(tar -xzf libeXosip2.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf libeXosip2.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_configuring "[$1] eXosip"
	(./configure --prefix=/usr &> /dev/null) || return 0
	
	display_info_compiling "[$1] eXosip"
	(make &> /dev/null) || return 0

	display_info_installing "[$1] eXosip"
	(make install &> /dev/null) || return 0

	return 1
}

install_ortp()
{
	cd $COMPILE_PATH

	display_info_extracting "[$1] oRTP"
	(tar -xzf ortp.tar.gz &> /dev/null) || return 0

	SUBPATH=`tar -tzf ortp.tar.gz |head -n 1`
	cd $COMPILE_PATH/$SUBPATH

	display_info_configuring "[$1] oRTP"
	(./configure --prefix=/usr &> /dev/null) || return 0
	
	display_info_compiling "[$1] oRTP"
	(make &> /dev/null) || return 0

	display_info_installing "[$1] oRTP"
	(make install &> /dev/null) || return 0

	return 1
}

# Installing items
install_zaptel "1 / 8"
if [[ "$?" == "0" ]]; then
	display_error "There was an error while installing zaptel.\nPlease contact $PRODUCT_NAME support"
fi

install_libpri "2 / 8"
if [[ "$?" == "0" ]]; then
	display_error "There was an error while installing libpri.\nPlease contact $PRODUCT_NAME support"
fi

install_wanpipe "3 / 8"
if [[ "$?" == "0" ]]; then
	display_error "There was an error while installing wanpipe.\nPlease contact $PRODUCT_NAME support"
fi

install_divas "4 / 8"
if [[ "$?" == "0" ]]; then
 	display_error "There was an error while installing divas4linux\nPlease contact $PRODUCT_NAME support"
fi

install_osip "5 / 8"
if [[ "$?" == "0" ]]; then
 	display_error "There was an error while installing oSIP.\nPlease contact $PRODUCT_NAME support"
fi

install_exosip "6 / 8"
if [[ "$?" == "0" ]]; then
 	display_error "There was an error while installing eXosip.\nPlease contact $PRODUCT_NAME support"
fi

install_ortp "7 / 8"
if [[ "$?" == "0" ]]; then
 	display_error "There was an error while installing oRTP.\nPlease contact $PRODUCT_NAME support"
fi

install_libzap "8 / 8"
if [[ "$?" == "0" ]]; then
 	display_error "There was an error while installing libzap.\nPlease contact $PRODUCT_NAME support"
fi

depmod -a

exit 0
