#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import re
import os.path

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-2]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from s2p import *

sys.path.append(CORE_PATH_SDK+'/python')

from s2p_base import *
from s2p_logger import *

S2P_CONF_SYSTEM		= xml_to_array(CORE_PATH_CONFIG+'/system.xml')

if os.path.isfile(CORE_PATH_CONFIG+'/drivers/zap.xml'):
	DRIVER_CONF = xml_to_array(CORE_PATH_CONFIG+'/drivers/zap.xml', 0)

else:
	s2p_log("configure.zap", "warn", "Missing configuration file, setting default values")
	
	DRIVER_CONF={
		'zap' : { 0: { '+': {
			'tonezone': {
				0: {
					'#': 'us'
				}
			}
		} } },
		'devices': { 0: { '+': {
		
		} } }
	}

def configure_tdm(code, n):
	default_conf={
		'media'				: 'fxo/fxs',
		'id'				: random_string(),
		'composing-timeout'	: 2,
		'tx-gain'			: 0.0,
		'rx-gain'			: 0.0,
		'echo-cancel'		: 'true',
		'reverse-polarity'	: 'none',
		'busy-detect'		: 4,
		'relax-dtmf'		: 'false',
	}

	if not DRIVER_CONF['devices'][0]['+'].has_key('item') or not DRIVER_CONF['devices'][0]['+']['item'].has_key(n):
		s2p_log("configure.zap", "info", "Detected new TDM card '"+code+"', setting to default values")
		dev_conf=default_conf
		
	else:
		dev_conf_tmp=xml_simplify_array(DRIVER_CONF['devices'][0]['+']['item'][n])

		if dev_conf_tmp.has_key('swtype') and len(dev_conf_tmp['swtype']):
			s2p_log("configure.zap", "warn", "Type mismatch for '"+code+"', did you change hardware config?")
			s2p_log("configure.zap", "warn", "Recovering TDM card '"+code+"', to default values")

			dev_conf=default_conf
		else:
			s2p_log("configure.zap", "info", "Detected an already existing TDM card '"+code+"'")
			dev_conf=dev_conf_tmp

	out ="<item>\n"

	for k in dev_conf.keys():
		if dev_conf[k] == {}:	out+="<"+k+" />\n"
		elif dev_conf[k] == "":	out+="<"+k+" />\n"
		else:					out+="<"+k+">"+str(dev_conf[k])+"</"+k+">\n"

	out+="</item>\n"

	return out
	
def configure_e1(code, n):
	default_conf={
		'media'				: 'e1',
		'id'				: random_string(),
		'composing-timeout'	: 0,
		'tx-gain'			: 0.0,
		'rx-gain'			: 0.0,
		'echo-cancel'		: 'true',
		'relax-dtmf'		: 'false',
		'crc4'				: 'true',
		'nodetype'			: 'net',
		'swtype'			: 'euroisdn',
		'layer'				: 'alaw',
		'overlap-dial'		: 'false',
		'timing'			: 0,
		'lbo'				: 0,
		'yellow'			: 'false'
	}

	if not DRIVER_CONF['devices'][0]['+'].has_key('item') or not DRIVER_CONF['devices'][0]['+']['item'].has_key(n):
		s2p_log("configure.zap", "info", "Detected new E1 card '"+code+"', setting to default values")
		dev_conf=default_conf
		
	else:
		dev_conf_tmp=xml_simplify_array(DRIVER_CONF['devices'][0]['+']['item'][n])

		if not dev_conf_tmp.has_key('swtype') or not len(dev_conf_tmp['swtype']):
			s2p_log("configure.zap", "warn", "Type mismatch for '"+code+"', did you change hardware config?")
			s2p_log("configure.zap", "warn", "Recovering E1 card '"+code+"', to default values")

			dev_conf=default_conf
		else:
			s2p_log("configure.zap", "info", "Detected an already existing E1 card '"+code+"'")
			dev_conf=dev_conf_tmp

	out ="<item>\n"

	for k in dev_conf.keys():
		if dev_conf[k] == {}:	out+="<"+k+" />\n"
		elif dev_conf[k] == "":	out+="<"+k+" />\n"
		else:					out+="<"+k+">"+str(dev_conf[k])+"</"+k+">\n"

	out+="</item>\n"

	return out
	
def configure_t1(code, n):
	default_conf={
		'media'				: 't1',
		'id'				: random_string(),
		'composing-timeout'	: 0,
		'tx-gain'			: 0.0,
		'rx-gain'			: 0.0,
		'echo-cancel'		: 'true',
		'relax-dtmf'		: 'false',
		'nodetype'			: 'net',
		'swtype'			: 'ni2',
		'layer'				: 'ulaw',
		'overlap-dial'		: 'false',
		'timing'			: 0,
		'lbo'				: 0
	}

	if not DRIVER_CONF['devices'][0]['+'].has_key('item') or  not DRIVER_CONF['devices'][0]['+']['item'].has_key(n):
		s2p_log("configure.zap", "info", "Detected new E1 card '"+code+"', setting to default values")
		dev_conf=default_conf
		
	else:
		dev_conf_tmp=xml_simplify_array(DRIVER_CONF['devices'][0]['+']['item'][n])

		if not dev_conf_tmp.has_key('swtype') or not len(dev_conf_tmp['swtype']):
			s2p_log("configure.zap", "warn", "Type mismatch for '"+code+"', did you change hardware config?")
			s2p_log("configure.zap", "warn", "Recovering T1 card '"+code+"', to default values")

			dev_conf=default_conf
		else:
			s2p_log("configure.zap", "info", "Detected an already existing T1 card '"+code+"'")
			dev_conf=dev_conf_tmp

	out ="<item>\n"

	for k in dev_conf.keys():
		if dev_conf[k] == {}:	out+="<"+k+" />\n"
		elif dev_conf[k] == "":	out+="<"+k+" />\n"
		else:					out+="<"+k+">"+str(dev_conf[k])+"</"+k+">\n"

	out+="</item>\n"

	return out

os.system(CORE_PATH_LIBEXEC+'/config_drivers/zap_wanpipe.py')

if not os.path.isdir('/proc/zaptel'): sys.exit(0)

xml_devices=""
device_n=0

while True:
	device_n=device_n+1

	if not os.path.isfile('/proc/zaptel/'+str(device_n)): break

	fp=open('/proc/zaptel/'+str(device_n), 'r')

	line=fp.readline()
	
	m=re.search(r'^[^:]+:\s+(?P<code>\w+/\d+)\s+"(?P<name>[^"]+)"', line)
	code=m.group('code')
	name=m.group('name')

	chan_count=0
	while True:
		line=fp.readline()
		if not line: break
		
		if re.search(r'\s*\d+\s+'+code, line): chan_count=chan_count+1

	if code.find('TDM') != -1:		mode = 'analog'
	elif code.find('WPE1') != -1:	mode = 'e1'
	elif code.find('WPT1') != -1:	mode = 't1'
	elif chan_count == 24:			mode = 't1'
	elif chan_count == 31:			mode = 'e1'
	else: mode = 'analog'

	fp.close()

	if mode == 'analog':	xml_devices+=configure_tdm(code, device_n-1)
	elif mode == 'e1':		xml_devices+=configure_e1(code, device_n-1)
	elif mode == 't1':		xml_devices+=configure_t1(code, device_n-1)

fp=open(CORE_PATH_CONFIG+'/drivers/zap.xml', 'w')
fp.write('<?xml version="1.0" encoding="utf-8"?>'+"\n")
fp.write("<config>\n")
fp.write("<zap>\n")
fp.write("<tonezone>"+DRIVER_CONF['zap'][0]['+']['tonezone'][0]['#']+"</tonezone>\n")

if not DRIVER_CONF['zap'][0]['+'].has_key('tonezone-custom'):
	DRIVER_CONF['zap'][0]['+']['tonezone-custom']={ 0: { '+': {} } }

	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dialtone']={ 0: { '#': "350+440" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['busy']={ 0: { '#': "480+620/500,0/500" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['ringtone']={ 0: { '#': "440+480/2000,0/4000" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['congestion']={ 0: { '#': "480+620/250,0/250" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['callwait']={ 0: { '#': "440/300,0/10000" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dialrecall']={ 0: { '#': "!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['recordtone']={ 0: { '#': "1400/500,0/15000" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['info']={ 0: { '#': "!950/330,!1400/330,!1800/330,0" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['stutter']={ 0: { '#': "!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,!350+440/100,!0/100,350+440" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['ringcadence']={ 0: { '#': "2000,4000" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dtmf-high-level']={ 0: { '#': "-10" } }
	DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dtmf-low-level']={ 0: { '#': "-10" } }
	

fp.write("<tonezone-custom>\n")
fp.write("<dialtone>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dialtone'][0]['#']+"</dialtone>\n")
fp.write("<busy>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['busy'][0]['#']+"</busy>\n")
fp.write("<ringtone>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['ringtone'][0]['#']+"</ringtone>\n")
fp.write("<congestion>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['congestion'][0]['#']+"</congestion>\n")
fp.write("<callwait>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['callwait'][0]['#']+"</callwait>\n")
fp.write("<dialrecall>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dialrecall'][0]['#']+"</dialrecall>\n")
fp.write("<recordtone>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['recordtone'][0]['#']+"</recordtone>\n")
fp.write("<info>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['info'][0]['#']+"</info>\n")
fp.write("<stutter>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['stutter'][0]['#']+"</stutter>\n")
fp.write("<ringcadence>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['ringcadence'][0]['#']+"</ringcadence>\n")
fp.write("<dtmf-high-level>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dtmf-high-level'][0]['#']+"</dtmf-high-level>\n")
fp.write("<dtmf-low-level>"+DRIVER_CONF['zap'][0]['+']['tonezone-custom'][0]['+']['dtmf-low-level'][0]['#']+"</dtmf-low-level>\n")
fp.write("</tonezone-custom>\n")

fp.write("</zap>\n")
fp.write("<devices>\n")
fp.write(xml_devices)
fp.write("</devices>\n")
fp.write("</config>\n")
fp.close()

os.system("chmod 666 /dev/zap/*")

