#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import re
import os.path

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-2]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from s2p import *

sys.path.append(CORE_PATH_SDK+'/python')

from s2p_base import *
from s2p_logger import *

S2P_CONF_SYSTEM		= xml_to_array(CORE_PATH_CONFIG+'/system.xml')

DRIVER_CONF={}
if os.path.isfile(CORE_PATH_CONFIG+'/drivers/zap_wanpipe.xml'):
	DRIVER_CONF = xml_to_array(CORE_PATH_CONFIG+'/drivers/zap_wanpipe.xml')

DRIVER_ZAP={}
if os.path.isfile(CORE_PATH_CONFIG+'/drivers/zap.xml'):
	DRIVER_ZAP = xml_to_array(CORE_PATH_CONFIG+'/drivers/zap.xml', 0)

devices=systempt("wanrouter hwprobe")
lines=devices.split('\n')

devices_info=[]
for line in lines:
	if not line: continue
	if not re.search(r'^\d+', line): continue

	fields=line.split(':')

	m=re.match("^(?P<devno>\d+)\s*\.\s*(?P<devname>[^\s]+)", fields[0])

	devno	= int(m.group('devno'))
	devname	= m.group('devname')

	if devname[0:3] == 'AFT':
		s2p_log("configure.zap", "info", "Found Sangoma '"+devname+"', will use wanrouter")
	else:
		s2p_log("configure.zap", "warn", "Found Sangoma '"+devname+"', this is not AFT")

	if re.search(r'A500', devname):
		s2p_log("configure.zap", "warn", "Found Sangoma '"+devname+"', this card is not supported")

	device_info={}
	for field in fields:
		try:
			[key, value]=field.split('=')
			
			device_info[trim(key)]=trim(value)

		except:
			1

	device_info['_N_']		= str(devno)
	device_info['_NAME_']	= devname

	if re.search(r'A10\d', devname):		device_info['_TYPE_'] = "WAN_AFT"
	elif  re.search(r'A20\d', devname):		device_info['_TYPE_'] = "WAN_AFT_ANALOG"
	elif  re.search(r'A40\d', devname):		device_info['_TYPE_'] = "WAN_AFT_ANALOG"
	else:
		s2p_log("configure.zap", "warn", "Found Sangoma '"+devname+"', this card is not supported")
		continue

	s2p_log("configure.zap", "info", "Configuring '"+devname+"' as "+device_info['_TYPE_'])
	devices_info.append(device_info)

# Restart wanpipe
os.system("/etc/init.d/wanrouter stop")
os.system("rm -f /etc/wanpipe/wanpipe?.conf &> /dev/null")

device_n=0
for device_info in devices_info:
	
	type_suffix=""
	if device_n > 0: type_suffix="_TE"+str(device_n)
	
	device_n=device_n+1

	wanpipe_cfg ="[devices]\n"
	wanpipe_cfg+="wanpipe"+device_info['_N_']+" = "+device_info['_TYPE_']+type_suffix+", Comment\n"
	wanpipe_cfg+="\n"
	
	wanpipe_cfg+="[interfaces]\n"
	wanpipe_cfg+="w1g"+device_info['_N_']+" = wanpipe"+device_info['_N_']+", , TDM_VOICE, Comment\n"
	wanpipe_cfg+="\n"

	wanpipe_cfg+="[wanpipe"+device_info['_N_']+"]\n"
	wanpipe_cfg+="CARD_TYPE         = AFT\n"
	wanpipe_cfg+="S514CPU           = A\n"
	wanpipe_cfg+="CommPort          = PRI\n"
	wanpipe_cfg+="AUTO_PCISLOT      = NO\n"
	wanpipe_cfg+="PCISLOT           = "+device_info['SLOT']+"\n"
	wanpipe_cfg+="PCIBUS            = "+device_info['BUS']+"\n"

	# PRI mode
	if device_info['_TYPE_'] == "WAN_AFT":

		item_code='item-'+device_info['SLOT']+'-'+device_info['BUS']+'-'+device_info['PORT']

		media	= ""
		frame	= ""
		clock	= ""
		code	= ""
		lbo		= ""

		if DRIVER_ZAP and DRIVER_CONF and DRIVER_CONF.has_key(item_code):
			n=int(DRIVER_CONF[item_code])

			# Try to get previously defined parameters
			if \
				DRIVER_ZAP['devices'][0]['+'].has_key('item') and \
				len(DRIVER_ZAP['devices'][0]['+']['item'].values())>n and \
				DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('media') \
			:
				media	= DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['media'][0]['#'].upper()

				if DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('coding') and DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['coding'][0]['#']:
					code	= DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['coding'][0]['#'].upper()
				else:
					code	= ""

				if DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('timing') and DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['timing'][0]['#']:
					if DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['timing'][0]['#'] == "0":
						timing="NORMAL"
					else:
						timing="MASTER"
						
				else:
					timing	= ""

				if media == "E1":
					lbos=['120OH', '75OH']
					
				else:
					lbos=['0db', '110-220ft', '220-330ft', '330-440ft', '550-660ft', '7.5dB', '15dB', '22.5dB']

				if DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('lbo') and DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['lbo'][0]['#']:
					lbo_n=int(DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['lbo'][0]['#'])
					if lbo_n >= len(lbos): lbo_n=0
					
					lbo=lbos[lbo_n]

				else:
					lbo	= ""

				if media == "E1":
					if DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('crc4') and DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['crc4'][0]['#']:
						if DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['crc4'][0]['#'] == 'true':
							frame	= "CRC4"
						else:
							frame	= "NCRC4"

					else:
						frame	= ""

				else:
					
					if DRIVER_ZAP['devices'][0]['+']['item'][n]['+'].has_key('framing') and DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['framing'][0]['#']:
						frame	= DRIVER_ZAP['devices'][0]['+']['item'][n]['+']['framing'][0]['#'].upper()
					else:
						frame	= ""

		if not media: media="E1"
		wanpipe_cfg+="FE_MEDIA          = "+media+"\n"

		if media == "E1":
			if not code: code="HDB3"
			if not frame: frame="NCRC4"
			
			wanpipe_cfg+="FE_LCODE          = "+code+"\n"
			wanpipe_cfg+="FE_FRAME          = "+frame+"\n"
			wanpipe_cfg+="TE_SIG_MODE       = CCS\n"
			
		else:
			if not code: code="B8ZS"
			if not frame: frame="ESF"
			
			wanpipe_cfg+="FE_LCODE          = "+code+"\n"
			wanpipe_cfg+="FE_FRAME          = "+frame+"\n"

		if not clock: clock="NORMAL"
		if not lbo:
			if media == "E1":	lbo="120OH"
			else:				lbo="0DB"
			
		wanpipe_cfg+="FE_LINE           = "+device_info['_N_']+"\n"
		wanpipe_cfg+="TE_CLOCK          = "+clock+"\n"
		wanpipe_cfg+="TE_REF_CLOCK      = 0\n"
		wanpipe_cfg+="TE_HIGHIMPEDANCE  = NO\n"
		wanpipe_cfg+="LBO               = "+lbo+"\n"
		wanpipe_cfg+="FE_TXTRISTATE     = NO\n"
		wanpipe_cfg+="TDMV_DCHAN        = 0\n"

	# Analog mode
	else:
		wanpipe_cfg+="FE_MEDIA          = FXO/FXS\n"
		wanpipe_cfg+="TDMV_LAW          = MULAW\n"
		wanpipe_cfg+="TDMV_OPERMODE     = FCC\n"
		wanpipe_cfg+="RM_BATTTHRESH     = 3\n"
		wanpipe_cfg+="RM_BATTDEBOUNCE   = 16\n"

	wanpipe_cfg+="MTU               = 1500\n"
	#wanpipe_cfg+="UDPPORT           = 9000\n"
	#wanpipe_cfg+="TTL               = 255\n"
	wanpipe_cfg+="IGNORE_FRONT_END  = NO\n"
	wanpipe_cfg+="TDMV_SPAN         = "+device_info['_N_']+"\n"
	wanpipe_cfg+="\n"

	wanpipe_cfg+="[w1g"+device_info['_N_']+"]\n"
	wanpipe_cfg+="ACTIVE_CH         = ALL\n"
	
	if device_info.has_key('HWEC') and int(device_info['HWEC']):
		wanpipe_cfg+="TDMV_HWEC         = YES\n"
		wanpipe_cfg+="TDMV_ECHO_OFF     = NO\n"
	else:
		wanpipe_cfg+="TDMV_HWEC         = NO\n"
		
	wanpipe_cfg+="\n"

	# Write wanpipe configuration
	fp=open("/etc/wanpipe/wanpipe"+device_info['_N_']+".conf", "w")
	fp.write(wanpipe_cfg)
	fp.close()

# Start wanpipe
if device_n: os.system("/etc/init.d/wanrouter start")

# Write device mapping
device_n=0
wp_device_n=0

map_file ="<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
map_file+="<config>\n"

while True:
	device_n	= device_n+1
	
	if not os.path.isfile('/proc/zaptel/'+str(device_n)): break

	fp=open('/proc/zaptel/'+str(device_n), 'r')

	line=fp.readline()
	
	m=re.search(r'^[^:]+:\s+(?P<code>\w+/\d+)\s+"(?P<name>[^"]+)"', line)
	code=m.group('code')
	name=m.group('name')

	fp.close()

	if code[0:2] == "WP" or code[0:2] == "WR":
		
		device_info = devices_info[wp_device_n]
		item_code='item-'+device_info['SLOT']+'-'+device_info['BUS']+'-'+device_info['PORT']
		
		map_file+="<"+item_code+">"+str(device_n-1)+"</"+item_code+">\n"
		wp_device_n	= wp_device_n+1

map_file+="</config>\n"

fp=open(CORE_PATH_CONFIG+'/drivers/zap_wanpipe.xml', "w")
fp.write(map_file)
fp.close()



