#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-2]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from s2p import *
from s2p_logger import *
from s2p_usock import *

S2P_CONF_SYSTEM	= xml_to_array(CORE_PATH_CONFIG+'/system.xml')
S2P_FORWARDING_ACCOUNT=os.environ.has_key('S2P_ISFORWARDING')

if S2P_FORWARDING_ACCOUNT:
	S2P_CONF_SKYPE	= xml_to_array(CORE_PATH_CONFIG+'/forwarding.xml', 0)
else:
	S2P_CONF_SKYPE	= xml_to_array(CORE_PATH_CONFIG+'/skype.xml', 0)

def session_s2p_log(level, message):
	global user_name
	global S2P_FORWARDING_ACCOUNT

	if S2P_FORWARDING_ACCOUNT:
		return s2p_log("fwd."+user_name, level, message)

	return s2p_log("sess."+user_name, level, message)

##########################################################################################
# Inizializzazione

user_name=os.environ['S2P_USER_SKYPE']
socket_client_file=os.environ['S2P_SOCKET_SKYPE']

# Ricerca utente
S2P_USER_INFO=0
for item in S2P_CONF_SKYPE['item'].values():
	if item['+']['skype'][0]['+']['username'][0]['#'] == user_name:
		S2P_USER_INFO=xml_simplify_array(item)
		break

if not S2P_USER_INFO: sys.exit(-1)

# Variabili di ambiente
sock_skype=s2p_usock()
sock_skype.connect(socket_client_file)
sock_skype.start()

##########################################################################################
# Scambio messaggi

skype_msg_counter=0

def ask_skype(message):
	global skype_msg_counter

	if message[0] == ':':
		return sock_skype.expect(message, '^:')

	if skype_msg_counter>=100: skype_msg_counter=0
	skype_msg_counter=skype_msg_counter+1

	message_key='#'+str(skype_msg_counter)
	message2=message_key+" "+message

	res=sock_skype.expect(message2, '^'+message_key+' ')
	return re.sub('^'+message_key+' ', '', res)

##########################################################################################
# Connessione Skype

def connect_skype():
	global user_name
	
	# Wait for Skype instance
	counter=0
	while 1:
		res=ask_skype(':SKSELECT:for Linux')
		m=re.match(r'^:SKCOUNT:(?P<value>\d+)$', res)

		if int(m.group('value')) > 0: break

		counter=counter+1
		if counter>30:
			session_s2p_log("error", "Skype did not start correctly")
			return -1

		time.sleep(1)

	time.sleep(2)

	session_s2p_log("trace", "Skype started correctly")

##########################################################################################
# Procedura di log-in

def do_login():
	global user_name
	
	# Send password
	ask_skype(':SKSELECT:for Linux')
	ask_skype(':SKSEND:'+S2P_USER_INFO['skype']['password'])
	ask_skype(':SKSENDSYM:Return')

	# Wait for log-in
	counter=0
	while 1:
		res=ask_skype(':SKSELECT:'+user_name+' - Skype')
		m=re.match(r'^:SKCOUNT:(?P<value>\d+)$', res)
		
		if int(m.group('value')) > 0: break
		
		counter=counter+1
		if counter>120:
			session_s2p_log("error", "Skype could not log-in")
			return -1

		time.sleep(1)

	session_s2p_log("trace", "Skype logged-in correctly")

	# Wait for session control
	counter=0
	while 1:
		ask_skype(':PING')
		
		res=ask_skype('NAME '+re.sub(r'\W', '', S2P_CONF_SYSTEM['hq']['sysname'].lower()))
		if res == "OK": return 0
		
		counter=counter+1
		if counter>5:
			session_s2p_log("error", "Skype refused API connection")
			return -1

		time.sleep(1)

	session_s2p_log("trace", "Skype attached correctly")

##########################################################################################
# Main

connect_skype()
if do_login(): sys.exit(2)

sys.exit(0)

