#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import os
import os.path
import string
import re
import time
import random
from xml.sax import saxutils

_selfpath=(os.path.dirname(os.path.realpath(__file__)).split('/'))[:-2]
_selfpath.append('pylibs')
_libspath='/'.join(_selfpath)
sys.path.append(_libspath)

from s2p import *
from s2p_logger import *
from s2p_usock import *

S2P_CONF_SYSTEM	= xml_to_array(CORE_PATH_CONFIG+'/system.xml')

def session_s2p_log(level, message):
	global user_name

	return s2p_log("sess."+user_name, level, message)

##########################################################################################
# Verifica dati in ingresso
if len(sys.argv) < 2:
	print "Usage: "+sys.argv[0]+" username"
	sys.exit(-1)

S2P_CONF_SKYPE	= xml_to_array(CORE_PATH_CONFIG+'/skype.xml', 0)

# Lettura dati
user_name=sys.argv[1]

# Validazione dati
if not re.match(r'^[\w\-_\.]+$', user_name) or re.match(r'^\.', user_name):
	print "Dangerous username"
	sys.exit(-1)

# Ricerca info utente
S2P_USER_INFO=0

c=0
for item in S2P_CONF_SKYPE['item'].values():
	c=c+1
	
	if item['+']['skype'][0]['+']['username'][0]['#'] == user_name:

		S2P_USER_INFO=xml_simplify_array(item)
		break

if not S2P_USER_INFO:
	print "Unknown user"
	sys.exit(-1)

if S2P_USER_INFO['enabled'] != "true":
	print "User is disabled"
	sys.exit(-1)

##########################################################################################
# VNC

if not S2P_USER_INFO['vnc'].has_key('screen') or not S2P_USER_INFO['vnc']['screen']:
	vnc_screens=systempt("netstat -tlnp|grep /"+os.path.basename(S2P_CONF_SYSTEM['bin']['vnc'])+"|grep ^tcp6|awk '{print $4}'|sed 's/:*//'").split("\n")
	S2P_USER_INFO['vnc']['screen']=1

	while 1:
		found=1
		for vnc_screen in vnc_screens:
			if not vnc_screen: continue

			if S2P_USER_INFO['vnc']['screen']==int(vnc_screen)-5900:
				S2P_USER_INFO['vnc']['screen']=S2P_USER_INFO['vnc']['screen']+1
				found=0
				break

		if found: break

##########################################################################################
# Assegnazione valori di ambiente

home=CORE_PATH_VH+'/'+user_name

myenv=os.environ
myenv['DISPLAY']=':'+str(S2P_USER_INFO['vnc']['screen'])
myenv['HOME']=home
myenv['USER']=S2P_CONF_SYSTEM['os']['username']
myenv['S2P_USER_SKYPE']=user_name

myenv['S2P_SOCKET_SKYPE']=S2P_CONF_SYSTEM['path']['socks']+'/connector-'+user_name+'.sock'

##########################################################################################
# Richiesta di riavvio della sessione

def do_restart():
	global user_name
	global S2P_CONF_SYSTEM

	s=s2p_usock()
	s.connect(S2P_CONF_SYSTEM['path']['socks']+'/sessions-manager.sock')
	s.set_xml_mode(True)

	out ="<message>\n"
	out+="<destination>"+saxutils.escape(user_name)+"</destination>\n"
	out+="<type>action</type>\n"
	out+="<action-name>session</action-name>\n"
	out+="<args>\n"
	out+="<arg name=\"mode\">restart</arg>\n"
	out+="</args>\n"
	out+="</message>"

	s.send(out)
	s.close()

##########################################################################################
# Avvio istanze

# Avvio VNC
if S2P_CONF_SYSTEM['tuning']['display'] == 'vfb':
	L = [
		S2P_CONF_SYSTEM['bin']['vfb'],
		":"+str(S2P_USER_INFO['vnc']['screen'])
	]
	pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

elif not S2P_USER_INFO['vnc']['password']:
	L = [
		S2P_CONF_SYSTEM['bin']['vnc'],
		":"+str(S2P_USER_INFO['vnc']['screen']),
		"-SecurityTypes", "none",
		"-geometry", S2P_CONF_SYSTEM['vnc']['geometry'],
		"-depth", S2P_CONF_SYSTEM['vnc']['depth'],
		"-desktop", user_name
	]
	pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

else:
	password=S2P_USER_INFO['vnc']['password']
	
	if len(password) < 6:
		for i in range(0, 6-len(password)):
			password=password+'x'
	
	os.system(CORE_PATH_LIBEXEC+'/vncpasswd.sh %s %s' % (home+'/.vnc/auth.pwd', password))
	
	L = [
		S2P_CONF_SYSTEM['bin']['vnc'],
		":"+str(S2P_USER_INFO['vnc']['screen']),
		#"-SecurityTypes", "VncAuth",
		"-PasswordFile", home+'/.vnc/auth.pwd',
		"-geometry", S2P_CONF_SYSTEM['vnc']['geometry'],
		"-depth", S2P_CONF_SYSTEM['vnc']['depth'],
		"-desktop", user_name
	]
	pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

s2p_register_pid('vnc-'+user_name, False, pid)
	

# Avvio wm
if S2P_CONF_SYSTEM['bin'].has_key('wm'):
	L=[S2P_CONF_SYSTEM['bin']['wm']]
	pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

	s2p_register_pid('wm-'+user_name, False, pid)

# Avvio skype
skype_myenv=myenv
skype_myenv['LD_PRELOAD']=CORE_PATH_LIBEXEC+"/shj.so:/lib/libdl.so.2"
L=[S2P_CONF_SYSTEM['bin']['skype']]
pid=os.spawnvpe(os.P_NOWAIT, L[0], L, skype_myenv)

s2p_register_pid('skype-'+user_name, False, pid)

# Avvio connector skype
L=[CORE_PATH_LIBEXEC+'/skype_connector']
pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

s2p_register_pid('connector-'+user_name, False, pid)

time.sleep(1)

# Avvio connector
L=[CORE_PATH_LIBEXEC+'/session/connect.py']
res=os.spawnvpe(os.P_WAIT, L[0], L, myenv)
if res==2:
	time.sleep(random.uniform(0, 5))
	session_s2p_log("warn", "Auto restarting session after failure")

	# In alcuni casi si incasina qualche file...
	os.system("rm -Rf "+home)

	do_restart()
	sys.exit(-1)

# Manager di sessione
L=[CORE_PATH_LIBEXEC+'/session/manage']
pid=os.spawnvpe(os.P_NOWAIT, L[0], L, myenv)

s2p_register_pid('manager-'+user_name, False, pid)

sys.exit(0)

