#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

import sys
import string
import os
import os.path
import re
import time
import random
import glob
import socket
import select
import exceptions
import threading
import thread

sys.path.append(os.path.dirname(os.path.realpath(__file__)))
from s2p import *

class s2p_usock(threading.Thread):
	def __init__(self, sock=None):
		threading.Thread.__init__(self)
		
		if sock is None:
			self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
		else:
			self.sock = sock
			
		self.sock.setblocking(0)

		self._stop=False
		self._filename=""

		self._xml_mode=False

		self._expect_c=0
		self._recv_buffer=[]
		self._expect_patterns={}
		self._new_packet	= threading.Condition()
		self._recv_lock		= threading.Lock()
		self._expc_lock		= threading.Lock()
		self._send_lock		= threading.Lock()

		self.set_xml_mode(False)

	# XML packet mode
	def set_xml_mode(self, status):
		self._xml_mode=status
		
		if status:	self._buffer_size=256
		else:		self._buffer_size=32

	# Accept incoming connection
	def accept(self):
		
		while True:
			ready_to_read, ready_to_write, in_error = select.select([self.sock], [], [self.sock], 1)
			
			if len(in_error):		return False
			if len(ready_to_read):	break
		
		(client_socket, foo)=self.sock.accept()
		
		res=s2p_usock(client_socket)
		res._filename=self._filename
		res.set_xml_mode(self._xml_mode)

		return res

	# Binding
	def bind(self, filename, bind_limit=1):
		self._filename=filename

		self.sock.bind(filename)
		self.sock.listen(bind_limit)

	# Chiusura socket
	def close(self):
		try:
			self.sock.close()
			self._stop=True

			# Rilascio lock expect
			for code in self._expect_patterns.keys():
				self._expect_patterns[code]['answer']=''
				self._expect_patterns[code]['lock'].acquire()
				self._expect_patterns[code]['lock'].notify()
				self._expect_patterns[code]['lock'].release()

			self._new_packet.acquire()
			self._new_packet.notify()
			self._new_packet.release()
		except:
			1	

	# Conenssione al socket
	def connect(self, filename):
		self._filename=filename

		counter=0

		while True:
			try:
				counter=counter+1
				if counter>10:
					self.close()
					sys.exit(-1)
				
				self.sock.connect(filename)
				break
				
			except:
				time.sleep(1)
				1

	# Invio diretto
	def send(self, msg):

		try:
			self._send_lock.acquire()
			totalsent = 0

			if self._xml_mode:	msg=msg+"\n<stop />\n"
			else:				msg=msg+"\n"

			while totalsent < len(msg):
				ready_to_read, ready_to_write, in_error = select.select([], [self.sock], [self.sock])
				
				try:
					sent = self.sock.send(msg[totalsent:].encode('UTF-8'))
				except:
					sent = self.sock.send(msg[totalsent:])

				if sent == 0: break
				totalsent = totalsent + sent

			self._send_lock.release()
			return totalsent

		except:
			self._send_lock.release()
			return -1

	def _on_packet(self, msg):
		if not len(msg): return

		self._expc_lock.acquire()

		# Verifica dei pattern di expect
		expect_match=False
		for code in self._expect_patterns.keys():

			#try:
			if not self._expect_patterns[code].has_key('pattern'): continue

			for j in range(0, len(self._expect_patterns[code]['pattern'])):
				expect_match=re.search(self._expect_patterns[code]['pattern'][j], msg)

				if expect_match:
					self._expect_patterns[code]['answer']=msg
					self._expect_patterns[code]['lock'].acquire()
					self._expect_patterns[code]['lock'].notify()
					self._expect_patterns[code]['lock'].release()
					break

			if expect_match: break

			#except:
			#	1

		self._expc_lock.release()

		# Immissione nel buffer di ricezione
		if not expect_match:
			self._new_packet.acquire()
			self._recv_buffer.append(msg)
			self._new_packet.notify()
			self._new_packet.release()

	# Verifica che un expect non sia rimasto in blocco troppo a lungo
	def _check_expect_timeout(self):

		while self.is_connected():
			self._expc_lock.acquire()
			
			for code in self._expect_patterns.keys():
				if time.time()-self._expect_patterns[code]['ts'] > 60:
					print "------------------------------------------------------"
					print "Code: "+code
					print self._expect_patterns[code]
					print "------------------------------------------------------"
					self._expect_patterns[code]['answer']=''
					self._expect_patterns[code]['lock'].acquire()
					self._expect_patterns[code]['lock'].notify()
					self._expect_patterns[code]['lock'].release()
					print "RELEASED"
					print "------------------------------------------------------"
					
			self._expc_lock.release()
			time.sleep(1)

	def run(self):

		buf=''

		thread.start_new_thread(self._check_expect_timeout, ())
		
		self.sock.setblocking(0)

		while True:
			try:
				chunk=''
				ready_to_read, ready_to_write, in_error = select.select([self.sock], [], [self.sock])

				try:
					chunk = self.sock.recv(self._buffer_size)
					buf=buf+chunk
					
				except: # Raised if resource is not avail (means no more data)
					break
			
				if not len(chunk): break
				
				if self._xml_mode:
					while re.search(r'<stop\s*/>', buf):
						[msg, buf]=re.split(r'[\n\r\s]*<stop\s*/>[\n\r\s]*', buf, 1)

						if not trim(msg): continue
						self._on_packet(msg)

				else:
					while re.search(r'[\r\n]', buf):
						[msg, buf]=re.split(r'[\n\r]', buf, 1)
						
						if not trim(msg): continue
						self._on_packet(msg)

			except:
				break

		#if buf: self._on_packet(buf)
		self.close() # sicuro?

	# Ritorna lo stato del canale
	def is_connected(self):
		if self.has_incoming(): return True
		return not self._stop

	# Leggi una linea dal buffer di ricezione
	def read(self):
		if not self.has_incoming(): return ''

		self._recv_lock.acquire()
		answer=self._recv_buffer[0]
		del(self._recv_buffer[0])
		self._recv_lock.release()
		
		return answer

	# Controlla se esistono dati in arrivo
	def has_incoming(self):
		if self.get_buflen(): return True

		return False
		
	def get_buflen(self):
		return len(self._recv_buffer)

	# Attende dati in arrivo
	def wait_for_incoming(self):
		if self.has_incoming(): return True
		if not self.is_connected(): return False

		try:
			self._new_packet.acquire()
			
			while True:
				self._new_packet.wait()
				
				if self.has_incoming():
					self._new_packet.release()
					return True
				
				if not self.is_connected():
					self._new_packet.release()
					return False

		except socket.error, msg:
			1

		self._new_packet.release()

		return False

	# Invia una richiesta e attende una risposta con delle caratteristiche
	def expect(self, message, pattern):

		# Creazione del pattern
		self._expc_lock.acquire()
		self._expect_c=self._expect_c+1
		self._expect_c=self._expect_c % 10000
		
		code=fix_digits(self._expect_c, 5)+random_string()

		if type(pattern) != list: pattern=[pattern]

		self._expect_patterns[code]={
			'message':	message,
			'pattern':	pattern,
			'lock':		threading.Condition(),
			'ts':		time.time(),
			'answer':	''
		}
		self._expc_lock.release()

		self._expect_patterns[code]['lock'].acquire() # Richiesta di blocco
		r=self.send(message)
		
		if r<0:
			self._expect_patterns[code]['lock'].release()
			del(self._expect_patterns[code])
			return ''

		# Attesa del rilascio del pattern dal ciclo run
		self._expect_patterns[code]['lock'].wait() # A questo punto il lock a' in attesa del primo release
		self._expect_patterns[code]['lock'].release() # Rilasciamo il lock precendente
		
		# Lettura della risposta
		answer=self._expect_patterns[code]['answer']

		# Rimozione del pattern
		self._expc_lock.acquire()
		del(self._expect_patterns[code])
		self._expc_lock.release()
		
		return answer
		
		
		