#!/usr/bin/env python

# Copyright and license: Please refer to LICENSE.TXT

########################################################################################
# Extra files

execfile(CORE_PATH_SDK+'/python/s2p_base/skype_call.py')
execfile(CORE_PATH_SDK+'/python/s2p_base/skype_message.py')

########################################################################################
# Skype management

class S2P_SKYPE(Thread):

	def __init__(self):
		Thread.__init__(self)
		
		self.sysconf=xml_to_array(CORE_PATH_CONFIG+'/system.xml')
		
		self._sock_sess=s2p_usock()
		self._sock_sess.set_xml_mode(True)
		self._sock_sess.connect(self.sysconf['path']['socks']+'/sessions-manager.sock')
		self._sock_sess.start()

		self._callback_functions={}
		self._calls={}					# Internal skype calls memory
		self._calls_garbage={}			# Internal skype calls garbage collector
		
		self._messages={}				# Internal chat messages memory

		self._events_history={}			# Keeps all the skype events of a call until it is destroyed

		self._garbage_timeout=15
		self._clean_lock=threading.Lock()
		
		self._stop=False
		
	def run(self):
		while self._sock_sess.is_connected():
			if self._sock_sess.wait_for_incoming():
				while self._sock_sess.get_buflen():
					self._on_message(self._sock_sess.read())

		self._sock_sess.close()

	def run3(self):
		while self._sock_sess.is_connected():
			if self._sock_sess.wait_for_incoming(): self._on_message(self._sock_sess.read())

		self._sock_sess.close()

	def get_classname(self):
		return "S2P_SKYPE"

	def stop(self):
		self._stop=True
		self._sock_sess.close()

		del self._sock_sess

	def _call_on_garbage(self, callid):
		self._calls_garbage[callid]=time.time()+self._garbage_timeout

	def _calls_garbage_clean(self):
		self._clean_lock.acquire()

		to_delete=[]
		
		for callid in self._calls_garbage.keys():
			if self._calls_garbage[callid] >= time.time(): continue

			if self._calls.has_key(callid):				del self._calls[callid]
			if self._events_history.has_key(callid):	del self._events_history[callid]

			to_delete.append(callid)

		for callid in to_delete: del self._calls_garbage[callid]

		self._clean_lock.release()

	def _parse_message(self, message):

		if not message:
			out={}
			out['type']='error'
			out['error-name']='empty-reply'
			out['args']={ 'arg': {} }

			return out

		try:
			res=xml_to_array(message, 0, True)
			
		except:
			print message
			
			out={}
			out['type']='error'
			out['error-name']='xml-error'
			out['args']={ 'arg': {} }

			return out

		out={}
		for k in res.keys():
			if k == 'args': continue
			out[k]=res[k][0]['#']

		out['args']={}

		if res.has_key('args'):
			if res['args'][0]['+'].has_key('arg'):
				for arg in res['args'][0]['+']['arg'].values():
				
					out['args'][arg['@']['name']]=arg['#']

		return out

	# DTMF event
	def _on_message_event_dtmf(self, parsed):
		callid=parsed['args']['call-id']
		dtmf=parsed['args']['code']

		# Junk events
		if not self._calls.has_key(callid): return

		# Invoke callback
		if self._callback_functions.has_key('dtmf'):
			self._callback_functions['dtmf'](self._calls[callid], dtmf)

	# Call event
	def _on_message_event_call(self, parsed):
		callid=parsed['args']['call-id']
		destroy_call=False

		if not self._events_history.has_key(callid): self._events_history[callid]=[]
		self._events_history[callid].append(parsed)

		# Junk events
		if \
			not self._calls.has_key(callid) and \
			parsed['event-name'] != 'call-incoming-ringing' and \
			parsed['event-name'] != 'call-incoming-overflow' and \
			parsed['event-name'] != 'call-outgoing-start' \
		: return

		# New call
		if \
			parsed['event-name'] == 'call-incoming-ringing' or \
			parsed['event-name'] == 'call-incoming-overflow' or \
			parsed['event-name'] == 'call-outgoing-start' \
		:
			if not self._calls.has_key(callid):
				self._calls_garbage_clean()
				self._calls[callid]=S2P_SKYPE_CALL(self, parsed['source'], callid)
			
			self._calls[callid].update(parsed['args'])

		# Status update
		elif parsed['event-name'] == 'call-status-change':
			self._calls[callid].update(parsed['args'])

		# Call closed
		elif parsed['event-name'] == 'call-incoming-terminated' or parsed['event-name'] == 'call-outgoing-terminated' or parsed['event-name'] == 'call-crashed':
			destroy_call=True

		# Invoke callback
		if self._callback_functions.has_key('call'):
			self._callback_functions['call'](self._calls[callid], parsed['event-name'])

		# Destroy the call if needed
		if destroy_call and self._calls.has_key(callid):			self._call_on_garbage(callid)

	# Message event
	def _on_message_event_message(self, parsed):
		messageid=parsed['args']['message-id']
		destroy_message=False

		# New message
		if not self._messages.has_key(messageid) and parsed['event-name'] != 'chatmessage-incoming-received': return

		if parsed['event-name'] == 'chatmessage-incoming-received':
			self._messages[messageid]=S2P_SKYPE_MESSAGE(self, parsed['source'], messageid)
			self._messages[messageid].update(parsed['args'])

		destroy_message=True

		# Invoke callback
		if self._callback_functions.has_key('message'):
			self._callback_functions['message'](self._messages[messageid], parsed['event-name'])

		# Destroy the call if needed
		#if destroy_message: del self._messages[messageid]
			

	# Event message
	def _on_message_event(self, parsed):
		
		# Invoke handled callbacks
		if parsed['event-name'][0:9] == 'call-dtmf':
			self._on_message_event_dtmf(parsed)
			
		if parsed['event-name'][0:5] == 'call-':
			self._on_message_event_call(parsed)

		if parsed['event-name'][0:12] == 'chatmessage-':
			self._on_message_event_message(parsed)

		# Invoke wildcard callback
		if self._callback_functions.has_key('raw-events'):
			self._callback_functions['raw-events'](parsed['source'], parsed['event-name'], parsed['args'])

	# New message received
	def _on_message(self, message):
		parsed=self._parse_message(message)

		if parsed['type'] == "event":
			self._on_message_event(parsed)

	def _compose_command(self, destination, command, args):

		msg_code=random_string()

		xml ="<message>\n";

		if type(destination) == list:
			xml+="<destinations>"+(','.join(destination))+"</destinations>\n";
		else:
			xml+="<destination>"+destination+"</destination>\n";
		
		xml+="<id>"+s2p_xml_escape(msg_code)+"</id>\n";
		xml+="<type>action</type>\n";
		xml+="<action-name>"+s2p_xml_escape(command)+"</action-name>\n";
		xml+="<args>\n";

		for k in args.keys():
			if args[k] == {}: args[k]=""
			xml+="<arg name=\""+k+"\">"+s2p_xml_escape(args[k])+"</arg>"

		xml+="</args>\n";
		xml+="</message>";

		return self._sock_send(xml, msg_code)

	def _sock_send(self, xml, msg_code):
		return self._sock_sess.expect(xml, r'<id>'+s2p_xml_escape(msg_code)+'</id>')

########################################################################################
# Callbacks

	def register_callback(self, key, callback_function):
		self._callback_functions[key]=callback_function

########################################################################################
# Misc functions

	def is_error(self, res):
		if res['type'] == "error": return True
		return False

########################################################################################
# Direct commands

	def command(self, destination, command, args):
		return self._parse_message(self._compose_command(destination, command, args))

########################################################################################
# Control commands

	def get_accounts_list(self):
		res=self._parse_message(self._compose_command("", "list-registered", {}))
		return res['args'].values()

########################################################################################
# Events history

	def get_events_history(self, call_id):
		if not self._events_history.has_key(call_id): return {}
		return self._events_history[call_id]

########################################################################################
# Call commands

	def get_calls(self):
		return self._calls

	def get_call(self, call_id):
		self._calls_garbage_clean()
		
		if not self._calls.has_key(call_id): return {}
		return self._calls[call_id]

	def call(self, source, args):
		res=self.command(source, "call-start", args)
		if self.is_error(res): return (False, res)

		# Set source (if multiple destinations)
		source=res['source']

		callid=res['args']['call-id']
		self._calls_garbage_clean()
		self._calls[callid]=S2P_SKYPE_CALL(self, source, callid)
		
		return (self._calls[callid], res)

########################################################################################
# Message commands

	def get_messages(self):
		return self._messages

	def message(self, module, args):
		res=self.command(module, "chatmessage-send", args)
		if self.is_error(res): return (False, res)

		messageid=res['args']['message-id']

		self._messages[messageid]=S2P_SKYPE_MESSAGE(self, module, messageid)

		return (self._messages[messageid], res)
			
		

